﻿using BinarySearch.Algorithm;
using BinarySearch.State;

namespace BinarySearch;

/// <summary>
/// Главная форма приложения для визуализации бинарного поиска.
/// </summary>
public partial class MainForm : Form
{
    private readonly StateStorage _stateStorage;
    private readonly Manager _manager;
    private SearchParameters _parameters;
    private BinarySearchVisualizer _visualizer;

    /// <summary>
    /// Конструктор главной формы.
    /// </summary>
    public MainForm()
    {
        InitializeComponent();
        _stateStorage = new StateStorage();
        _manager = new Manager(_stateStorage);
        _parameters = new SearchParameters();
        _visualizer = new BinarySearchVisualizer(this, _parameters.Array);
        buttonStepForward.Enabled = false;
        buttonStepBack.Enabled = false;
        buttonStart.Enabled = false;
    }

    /// <summary>
    /// Обработчик события добавления данных.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonAddData_Click(object sender, EventArgs e)
    {
        InitDataForm initDataForm = new();
        initDataForm.InitParametersEvent += InitParametersEventHandler;
        initDataForm.ShowDialog();
    }

    /// <summary>
    /// Обработчик события инициализации параметров поиска.
    /// </summary>
    /// <param name="searchParameters"></param>
    private void InitParametersEventHandler(SearchParameters searchParameters)
    {
        _parameters = new SearchParameters(searchParameters.Array, searchParameters.Target);
        _visualizer = new BinarySearchVisualizer(this, _parameters.Array);
        _visualizer.UpdateFlowLayoutPanel();
        labelTarget.Text = searchParameters.Target.ToString();
        buttonStart.Enabled = true;
    }

    /// <summary>
    /// Обработчик события запуска бинарного поиска.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonStart_Click(object sender, EventArgs e)
    {
        _stateStorage.Reset();
        _visualizer.UpdateFlowLayoutPanel();
        _manager.ExecuteBinarySearch(_parameters);
        toolStripStatusLabelElems.Text = _parameters?.Array?.Count.ToString();
        toolStripProgressBarSteps.Maximum = _stateStorage.GetStatesCount();
        textBoxSteps.Text = _stateStorage.GetStatesCount().ToString();
        toolStripProgressBarSteps.Value = 0;
        textBoxStep.Text = 0.ToString();
        toolStripMenuItemSave.Enabled = true;
        buttonStepForward.Enabled = true;
        buttonStepBack.Enabled = false;
    }

    /// <summary>
    /// Обработчик события шага вперед бинарного поиска.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonStepForward_Click(object sender, EventArgs e)
    {
        SearchState? searchState = _stateStorage.GetNextState();
        toolStripProgressBarSteps.Value = _stateStorage.GetCurrentIndex()+1;
        textBoxStep.Text = (_stateStorage.GetCurrentIndex() + 1).ToString();
        SearchState? searchStateCheck = _stateStorage.GetNextState();
        if (searchStateCheck == null && searchState != null)
        {
            buttonStepForward.Enabled = false;
            if (searchState.TargetIndex != null && searchState.TargetIndex >= 0)
            {
                MessageBox.Show($"Число {searchState.Target} найдено с индексом: {searchState.TargetIndex}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
            } else if (searchState.TargetIndex != null && searchState.TargetIndex < 0)
            {
                MessageBox.Show($"Число {searchState.Target} не найдено", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            _visualizer.Visualize(searchState);
            return;
        }
        searchState = _stateStorage.GetPreviousState();
        if (searchState == null)
        {
            return;
        }
        calculateInfo(searchState);
        _visualizer.Visualize(searchState);
        buttonStepForward.Enabled = true;
        buttonStepBack.Enabled = true;
    }

    /// <summary>
    /// Обработчик события шага назад бинарного поиска.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonStepBack_Click(object sender, EventArgs e)
    {
        SearchState? searchState = _stateStorage.GetPreviousState();
        if (searchState == null)
        {
            return;
        }
        toolStripProgressBarSteps.Value = _stateStorage.GetCurrentIndex()+1;
        textBoxStep.Text = (_stateStorage.GetCurrentIndex() + 1).ToString();
        calculateInfo(searchState);
        _visualizer.Visualize(searchState);
        buttonStepForward.Enabled = true;
        buttonStepBack.Enabled = true;
    }

    private void calculateInfo(SearchState state)
    {
        int currentNumber = state.Array[state.Mid];
        int target = state.Target;
        textBoxCurrent.Text = currentNumber.ToString();
        textBoxTarget.Text = target.ToString();
        if (target < currentNumber)
        {
            labelConditional.Text = "больше";
            labelResult.Text = "берём левый подмассив";
        } else if (target > currentNumber)
        {
            labelConditional.Text = "меньше";
            labelResult.Text = "берём правый подмассив";
        } else
        {
            labelConditional.Text = "равно";
            labelResult.Text = "берём это значение";
        }
    }

    /// <summary>
    /// Обработчик события открытия формы информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonInfo_Click(object sender, EventArgs e)
    {
        InfoForm infoForm = new();
        infoForm.ShowDialog();
    }

    /// <summary>
    /// Обработчик события сохранения состояния в файл.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void toolStripMenuItemSave_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_stateStorage.SaveToFile(saveFileDialog.FileName))
            {
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Не сохранилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

    /// <summary>
    /// Обработчик события загрузки состояния из файла.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void toolStripMenuItemLoad_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_stateStorage.LoadFromFile(openFileDialog.FileName))
            {
                SearchState? state = _stateStorage.GetNextState();
                if (state != null)
                {
                    _parameters = new SearchParameters(state.Array, state.Target);
                    _visualizer = new BinarySearchVisualizer(this, _parameters.Array);
                    _visualizer.UpdateFlowLayoutPanel();
                    labelTarget.Text = state.Target.ToString();
                    _stateStorage.GetPreviousState();
                }
                buttonStart.Enabled = true;
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}