﻿using System.Runtime.Serialization.Formatters.Binary;
using System.Text.Json;

namespace BinarySearch.State;

/// <summary>
/// Класс-Хранилище
/// </summary>
public class StateStorage
{
    private List<SearchState> states;
    private int currentIndex;

    /// <summary>
    /// Конструктор класса-хранилища
    /// </summary>
    public StateStorage()
    {
        states = new List<SearchState>();
        currentIndex = -1;
    }

    /// <summary>
    /// Получение количества элементов хранилище
    /// </summary>
    /// <returns></returns>
    public int GetStatesCount() { return states.Count; }

    /// <summary>
    /// Получения актуального индекса
    /// </summary>
    /// <returns></returns>
    public int GetCurrentIndex() { return currentIndex; }

    /// <summary>
    /// Добавление состояния в хранилище
    /// </summary>
    /// <param name="state"></param>
    public void AddState(SearchState state)
    {
        states.Add(state);
    }

    /// <summary>
    /// Очистка хранилища
    /// </summary>
    public void Clear()
    {
        states.Clear();
        Reset();
    }

    /// <summary>
    /// Сброс актуального индекса
    /// </summary>
    public void Reset()
    {
        currentIndex = -1;
    }

    /// <summary>
    /// Получение следующего состояния
    /// </summary>
    /// <returns></returns>
    public SearchState? GetNextState()
    {
        if (currentIndex+1 < states.Count)
        {
            currentIndex++;
            return states[currentIndex];
        }
        else
        {
            return null;
        }
    }

    /// <summary>
    /// Получение предыдущего состояния
    /// </summary>
    /// <returns></returns>
    public SearchState? GetPreviousState()
    {
        if (currentIndex > 0 && currentIndex < states.Count)
        {
            currentIndex--;
            return states[currentIndex];
        }
        else
        {
            return null;
        }
    }

    /// <summary>
    /// Проверка на то дошли ли мы до конца хранилища
    /// </summary>
    /// <returns></returns>
    public bool IsTraversalComplete()
    {
        return currentIndex >= states.Count - 1;
    }

    /// <summary>
    /// Сохранение хранилища в бинарный файл или JSON файл
    /// </summary>
    /// <param name="filename"></param>
    /// <returns></returns>
    public bool SaveToFile(string filename)
    {
        try
        {
            string extension = Path.GetExtension(filename).ToLower();

            if (extension == ".bin")
            {
                // Сохранение в бинарный формат
                BinaryFormatter formatter = new BinaryFormatter();
                using FileStream fs = new FileStream(filename, FileMode.Create);
                #pragma warning disable SYSLIB0011
                formatter.Serialize(fs, states);
                #pragma warning restore SYSLIB0011
            }
            else if (extension == ".json")
            {
                // Сохранение в формат JSON
                string json = JsonSerializer.Serialize(states);
                File.WriteAllText(filename, json);
            }
            else
            {
                // Неподдерживаемое расширение файла
                Console.WriteLine("Unsupported file extension");
                return false;
            }

            return true;
        }
        catch (Exception ex)
        {
            Console.WriteLine($"Error saving file: {ex.Message}");
            return false;
        }
    }

    /// <summary>
    /// Загрузка хранилища из бинарного файла или JSON файла
    /// </summary>
    /// <param name="filename"></param>
    /// <returns></returns>
    public bool LoadFromFile(string filename)
    {
        try
        {
            string extension = Path.GetExtension(filename).ToLower();

            if (extension == ".bin")
            {
                // Загрузка из бинарного файла
                BinaryFormatter formatter = new BinaryFormatter();
                using FileStream fs = new FileStream(filename, FileMode.Open);
                #pragma warning disable SYSLIB0011
                states = (List<SearchState>)formatter.Deserialize(fs);
                #pragma warning restore SYSLIB0011
            }
            else if (extension == ".json")
            {
                // Загрузка из файла в формате JSON
                string json = File.ReadAllText(filename);
                states = JsonSerializer.Deserialize<List<SearchState>>(json);
            }
            else
            {
                // Неподдерживаемое расширение файла
                Console.WriteLine("Unsupported file extension");
                return false;
            }

            return true;
        }
        catch (Exception ex)
        {
            Console.WriteLine($"Error loading file: {ex.Message}");
            return false;
        }
    }
}
